/**
 * GZipUtil.java 2013-10-26
 * 
 * 天津云翔联动科技有限公司(c) 1995 - 2014 。
 * http://www.soaring-cloud.com.cn
 *
 */
package com.soaringcloud.library.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import android.util.Log;

/**
 * <b>GZipUtil。</b>
 * <p><b>详细说明：</b></p>
 * <!-- 在此添加详细说明 -->
 * 无。
 * <p><b>修改列表：</b></p>
 * <table width="100%" cellSpacing=1 cellPadding=3 border=1>
 * <tr bgcolor="#CCCCFF"><td>序号</td><td>作者</td><td>修改日期</td><td>修改内容</td></tr>
 * <!-- 在此添加修改列表，参考第一行内容 -->
 * <tr><td>1</td><td>wancheng</td><td>2013-10-26 上午9:35:31</td><td>建立类型</td></tr>
 * 
 * </table>
 * @version 1.0
 * @author wancheng
 * @since 1.0
 */
public class GZipUtil {
	
	private static final String TAG = GZipUtil.class.getName();
	private static GZipUtil GU = null;
	public static final int BUFFER = 1024;
	public static final String EXT = ".gz";
	
	private GZipUtil(){
		
	}

	public static GZipUtil getInstance(){
		if(null == GU){
			synchronized (GZipUtil.class) {
				if(null == GU) GU = new GZipUtil();
			}
		}
		return GU;
	}
	
	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 数据压缩
	 * @param data
	 * @return
	 */
	public byte[] compress(byte[] data){
		byte[] output = null;
		ByteArrayInputStream bais = null;
		ByteArrayOutputStream baos = null;
		try{
			bais = new ByteArrayInputStream(data);
			baos = new ByteArrayOutputStream();
			compress(bais, baos);
			output = baos.toByteArray();
			baos.flush();
			baos.close();
			bais.close();
		}catch (IOException e) {
			e.printStackTrace();
			Log.e(TAG, e.toString());
		}finally{
			try{
				if(null != bais) bais.close();
				if(null != baos) baos.close();
			}catch (IOException e) {
				Log.e(TAG, e.toString());
			}
		}
		return output;
	}

	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件压缩
	 * @param file
	 * @return 压缩后File
	 */
	public File compress(File file){
		return compress(file, true);
	}

	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件压缩
	 * @param file
	 * @param delete 是否删除原始文件
	 * @return 压缩后File
	 */
	public File compress(File file, boolean delete){
		File zipFile = null;
		FileInputStream fis = null;
		FileOutputStream fos = null;
		try{
			zipFile = new File(file.getCanonicalPath() + EXT);
			fis = new FileInputStream(file);
			fos = new FileOutputStream(zipFile);
			compress(fis, fos);
			fis.close();
			fos.flush();
			fos.close();
			if (delete) {
				file.delete();
			}
		} catch (Exception e) {
			e.printStackTrace();
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != fis) fis.close();
				if(null != fos) fos.close();
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
		return zipFile;
	}

	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 数据压缩
	 * @param is
	 * @param os
	 */
	public void compress(InputStream is, OutputStream os){
		GZIPOutputStream gos = null;
		try{
			gos = new GZIPOutputStream(os);
			int count;
			byte data[] = new byte[BUFFER];
			while ((count = is.read(data, 0, BUFFER)) != -1) {
				gos.write(data, 0, count);
			}
			gos.finish();
			gos.flush();
			gos.close();
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try {
				if(null != gos) gos.close();
			} catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
	}

	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件压缩
	 * @param path
	 * @return 压缩后路径
	 */
	public String compress(String path){
		return compress(path, true);
	}

	/**
	 * 
	 * <b>compress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件压缩
	 * @param path
	 * @param delete 是否删除原始文件
	 * @return 压缩后路径
	 */
	public String compress(String path, boolean delete) {
		File zipFile = compress(new File(path), delete);
		if(null!=zipFile) return zipFile.getPath();
		return null;
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 数据解压缩
	 * @param data
	 * @return
	 */
	public byte[] uncompress(byte[] data){
		ByteArrayInputStream bais = null;
		ByteArrayOutputStream baos = null;
		try{
			bais = new ByteArrayInputStream(data);
			baos = new ByteArrayOutputStream();
			uncompress(bais, baos);
			data = baos.toByteArray();
			baos.flush();
			baos.close();
			bais.close();
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != bais) bais.close();
				if(null != baos) baos.close();
			}catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
		return data;
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件解压缩
	 * @param file
	 * @return 解压后File
	 */
	public File uncompress(File file){
		return uncompress(file, true);
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件解压缩
	 * @param file
	 * @param delete 是否删除原始文件
	 */
	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件解压缩
	 * @param file
	 * @param delete 是否删除原始文件
	 * @return 解压后File
	 */
	public File uncompress(File file, boolean delete){
		File unzipFile = null;
		FileInputStream fis = null;
		FileOutputStream fos = null;
		try{
			unzipFile = new File(file.getCanonicalPath().replace(EXT, ""));
			fis = new FileInputStream(file);
			fos = new FileOutputStream(unzipFile);
			uncompress(fis, fos);
			fis.close();
			fos.flush();
			fos.close();
			if (delete) {
				file.delete();
			}
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != fis) fis.close();
				if(null != fos) fos.close();
			} catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
		return unzipFile;
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 数据解压缩
	 * @param is
	 * @param os
	 */
	public void uncompress(InputStream is, OutputStream os){
		GZIPInputStream gis = null;
		try{
			gis = new GZIPInputStream(is);
			int count;
			byte data[] = new byte[BUFFER];
			while ((count = gis.read(data, 0, BUFFER)) != -1) {
				os.write(data, 0, count);
			}
			gis.close();
		}catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != gis) gis.close();
			} catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件解压缩
	 * @param path
	 * @return 解压后路径
	 */
	public String uncompress(String path) {
		return uncompress(path, true);
	}

	/**
	 * 
	 * <b>uncompress。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 文件解压缩
	 * @param path
	 * @param delete 是否删除原始文件
	 * @return 解压后路径
	 */
	public String uncompress(String path, boolean delete) {
		File unzipFile = uncompress(new File(path), delete);;
		if(null != unzipFile) return unzipFile.getPath();
		return null;
	}

	/**
	 * 
	 * <b>compressStr。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 压缩字符串
	 * @param str
	 * @return 压缩后字符串
	 */
	public String compressStr(String str) {
		if (str == null || str.length() == 0) {
			return str;
		}
		ByteArrayOutputStream baos = null;
		GZIPOutputStream gos = null;
		String result = null;
		try{
			baos = new ByteArrayOutputStream();
			gos = new GZIPOutputStream(baos);
			gos.write(str.getBytes());
			gos.close();
			result = baos.toString("ISO-8859-1");
			baos.close();
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != gos) gos.close();
				if(null != baos) baos.close();
			}catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
		return result;
	}

	/**
	 * 
	 * <b>uncompressStr。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 解压缩字符串
	 * @param str
	 * @return 解压后字符串
	 */
	public String uncompressStr(String str) {
		if (str == null || str.length() == 0) {
			return str;
		}
		ByteArrayOutputStream baos = null;
		ByteArrayInputStream bais = null;
		GZIPInputStream gis = null;
		String result = null;
		try{
			baos = new ByteArrayOutputStream();
			bais = new ByteArrayInputStream(str.getBytes("ISO-8859-1"));
			gis = new GZIPInputStream(bais);
			byte[] buffer = new byte[256];
			int n;
			while ((n = gis.read(buffer)) >= 0) {
				baos.write(buffer, 0, n);
			}
			result = baos.toString();
			gis.close();
			bais.close();
			baos.close();
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		} finally {
			try{
				if(null != gis) gis.close();
				if(null != bais) bais.close();
				if(null != baos) baos.close();
			}catch (Exception e) {
				Log.e(TAG, e.toString());
			}
		}
		return result;
	}
}
